#ifndef UtilityH
#define UtilityH

#include <ShlWApi.h>
#include <vector>
#include <algorithm>
using namespace std;

class Persist;

class Bool {
private:
	bool m_value;

public:
	Bool(bool value) : m_value(value) {}
	Bool(BOOL value) : m_value(value != FALSE) {}
	operator bool() {return m_value;}
	operator BOOL() {return m_value ? TRUE : FALSE;}
};

template<class T>
class Vector : public vector<T> {
protected:
	const T& Item(size_t i) const {return (*this)[i];}
	T& Item(size_t i) {return (*this)[i];}

public:
	iterator find(const T& item) {return std::find(begin(), end(), item);}
	const_iterator find(const T& item) const {return std::find(begin(), end(), item);}
};

class StringVector : public Vector<CString> {
public:
	void GetLineRange(long line, long& start, long &end) const;
	long GetTotalLength() const;
	void LineFromChar(long& lineOffset, long& lineNum, long pos) const;
	CString GetString(const char* separator) const;
	long FindString(const CString& string, bool matchCase) const;
	bool HasString(const CString& string, bool matchCase) const {return FindString(string, matchCase) >= 0;}
	void RemoveAllOccurrencesOf(const CString& string, bool matchCase);
	void RemoveDuplicates(bool matchCase);
	CString Concat(const CString& separator);
};

#define VERSUCC(x) VERIFY(SUCCEEDED(x))

template<class T>
T Min(const T& val1, const T& val2) {
	return val1 < val2 ? val1 : val2;
}

template<class T>
T Max(const T& val1, const T& val2) {
	return val1 > val2 ? val1 : val2;
}

//returns true if b1 and b2 are either both true or both false
template<class Bool1, class Bool2>
bool BoolEq(Bool1 b1, Bool2 b2) {
	return (b1 && b2) || (!b1 && !b2);
}

template<class T>
void MemSet(T& data, BYTE value) {
	memset(&data, value, sizeof(data));
}

template<class T>
void MemClear(T& data) {
	MemSet(data, 0x00);
}

template<class T>
void InitSized(T& data) {
	MemClear(data);
	data.cbSize = sizeof(data);
}

extern LPCTSTR EmptyStr;
extern LPCTSTR ErrorStr;

typedef BYTE VirtualKey;
typedef BYTE KeyModifiers;
const KeyModifiers kmNone = 0x0;
const KeyModifiers kmShift = 0x1;
const KeyModifiers kmCtrl = 0x2;
const KeyModifiers kmAlt = 0x4;

inline bool IsKeyDown(VirtualKey vk) {return (GetKeyState(vk) & 0x8000) != 0;}
bool IsAsyncKeyDown(VirtualKey vk);
KeyModifiers GetKeyModifiers();
CString KeyToString(VirtualKey vk);
CString KeyAndModifiersToString(VirtualKey vk, KeyModifiers mods);

//for conversion between KeyModifiers and the modifiers for a CHotKeyCtrl
WORD ToHotKeyMods(KeyModifiers mods);
KeyModifiers FromHotKeyMods(WORD mods);

inline CString CharToStr(TCHAR ch) {return ch == NULL ? EmptyStr : CString(ch);}
inline bool IsVarChar(TCHAR ch) {return _istalnum(ch) || ch == '_';}

inline int StrCmpEx(LPCTSTR s1, LPCTSTR s2, bool matchCase) {
	return matchCase ? StrCmp(s1, s2) : StrCmpI(s1, s2);
}

inline bool ChrCmp(TCHAR ch1, TCHAR ch2, bool ignoreCase) {
	return ignoreCase ? _totlower(ch1) == _totlower(ch2) : ch1 == ch2;
}

void AppendString(CString& string, const CString&, LPCTSTR separator);
long GetNumCharsFit(CDC& dc, LPCTSTR string, long length, long maxWidth);
long GetPrevWordPos(const CString& string, long pos, bool eatSpaces);
long GetNextWordPos(const CString& string, long pos, bool eatSpaces);
bool IsValidIdentifier(const CString& string);
long FindFirstDiff(LPCTSTR s1, LPCTSTR s2);
void GetLogFont(const CFont& font, LOGFONT& lf);
HFONT CloneFont(const CFont& font);
bool FontsEqual(const CFont& font1, const CFont& font2);
long CountChar(const CString& string, TCHAR ch);
void MakeTitleCase(CString& string);
void MakeToggleCase(CString& string);
CString DupChar(TCHAR ch, long length);
void StringListToStringVector(const CString& list, LPCTSTR separator, StringVector& vec);
void UnescapeChars(CString& string);
CString GetErrorMessage(DWORD errorCode);
TCHAR GetMnemonic(const CString& string);
inline CString Quote(const CString& string) {return TEXT("\"") + string + TEXT("\"");}

enum RoundKinds {rkUp, rkDown, rkOff};
long Div(long num1, long num2, RoundKinds rk);

HMENU LoadSubMenu(int menuId, int pos);
void SetMenuItemEnabled(HMENU hMenu, UINT itemId, bool enable, bool byPos = false);
void SetSubMenu(HMENU menu, int subPos, HMENU subMenu);
int FindMenuItemPosById(HMENU hMenu, UINT id);
void CleanupMenuItemName(CString& name);

bool GetClipboardString(CString& text, HWND hwnd, bool convertFromCRLF);
bool SetClipboardString(const CString& text, HWND hwnd, bool convertToCRLF);

CString GetSystemTimeString(bool longTime);
CString GetSystemDateString(bool longDate);

enum NewLineTypes {nlCRLF, nlCR, nlLF};//CR is '\r' and LF is '\n'

void ConvertCRLFtoLF(CString& string);
void ConvertCRLFtoLF(LPTSTR dest, LPCTSTR src);

CString ConvertLFtoCRLF(const CString& string, long lineCount = -1);
void ConvertLFtoCRLF(LPTSTR dest, LPCTSTR src);

void ConvertLFtoCR(CString& string);
void ConvertLFtoCR(LPTSTR dest, LPCTSTR src);

void ConvertAnyToLF(CString& string);
void ConvertAnyToLF(LPTSTR dest, LPCTSTR src);

CString FileToString(const CString& fileName, bool convertNewLinesToLFs = true);
void StringToFile(const CString& fileName, const CString& data);
bool FileExists(const CString& name);
bool AbsolutePathExists(const CString& path);
bool IsLocalFile(const CString& path);
CString GetProgramFilesPath();
CString GetInternetExplorerPath();
CString GetModulePath();
CString GetModuleDir();
CString MakePath(const CString& dir, const CString& fileName);
CString GetDir(const CString& path);
CString GetFileNameWithExt(const CString& path);
bool HasExt(const CString& path);
CString GetFileExt(const CString& path);
static CString InvalidFileNameChars = TEXT("\\/*:?\"<>|");
bool IsValidFileName(const CString& fileName);
static CString InvalidFileExtChars = InvalidFileNameChars + '.';
bool IsValidFileExt(const CString& ext);
CString GetInvalidExtErrorMessage(const CString& ext);
FILETIME GetFileModifyTime(const CString& fileName);
bool SearchForFile(CString& fileName);
void BreakPath(const CString& path, CString& dir, CString& name, CString& ext);
CString ExpandEnvironmentStrings(const CString& string);
bool BrowseForFolder(HWND hwnd, CString& path, UINT flags = 0);
HRESULT CreateLink(LPCSTR lpszPathObj, LPCSTR lpszPathLink, LPCSTR lpszDesc);
void GetSpecialFolderPath(int folder, TCHAR path[]);
bool CreateDirPath(const CString& dir);
bool UninstallFile(const CString& path);
long GetNumDigits(long num);
void FindAllFiles(const CString& fileName, StringVector& vec, bool includeFullPath);

void MoveWindowBy(CWnd& wnd, int dx, int dy);
bool FindItemByData(const CComboBox& comboBox, DWORD itemData, int& index);
void SetListBoxItemText(CListBox& listBox, int index, const CString& text);
void StringVectorToComboBox(CComboBox& comboBox, const StringVector& vec);
RECT GetScreenRect();
void LinkifyText(CRichEditCtrl& richEdit, LPCTSTR text);

//I define both ffMatchCase and ffIgnoreCase as non-zero so you have to be explicit and
//pass one (i.e. no default on case sensetivity).
typedef BYTE FindFlags;
const FindFlags ffMatchCase = 0x1;
const FindFlags ffIgnoreCase = 0x2;
const FindFlags ffWholeWord = 0x4;
const FindFlags ffReverse = 0x8;

long FindSubstring(const CString& search, const CString& find, FindFlags flags, long start = 0, long end = -1);
long CyclicFindSubstring(const CString& search, const CString& find, FindFlags flags, long start, long end);
long ReplaceSubstring(CString& string, const CString& find, const CString& replace, FindFlags flags, long start = 0, long maxReplacments = -1);

class StringIterator {
public:
	virtual long GetNumStrings() const = 0;
	virtual const CString& GetNthString(long i) const = 0;
};

long FindMatch(const StringIterator& vec, const CString& string, bool matchCase);

enum StringToken {stIdentifier, stWhiteSpace, stSymbol};
StringToken GetNextToken(const CString& string, long start, CString& token);

class WaitCursor {
private:
	HCURSOR mOldCursor;

public:
	WaitCursor() {
		mOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	}

	~WaitCursor() {
		SetCursor(mOldCursor);
	}
};

struct Accel : ACCEL {
	Accel();
	Accel(VirtualKey vkey, KeyModifiers mods, WORD command);

	VirtualKey GetVirtualKey() const {return (VirtualKey)key;}
	KeyModifiers GetModifiers() const;
	void DoPersist(Persist& p);
};

class AccelTable {
private:
	HACCEL m_handle;
	vector<Accel> m_accels;

	void CreateIfNecessary() {
		if (m_handle == NULL)
			m_handle = CreateAcceleratorTable(m_accels.begin(), m_accels.size());
	}

	void Destroy() {
		if (m_handle != NULL) {
			VERIFY(DestroyAcceleratorTable(m_handle));
			m_handle = NULL;
		}
	}

	vector<Accel>::const_iterator FindAccelByCmd(UINT cmd) const;

public:
	AccelTable() : m_handle(NULL) {}
	AccelTable(const AccelTable& rhs) {*this = rhs;}
	~AccelTable() {Destroy();}

	AccelTable& operator=(const AccelTable& rhs) {
		m_accels = rhs.m_accels;
		Destroy();
		return *this;
	}

	operator HACCEL() {
		CreateIfNecessary();
		return m_handle;
	}

	void AddAccel(const Accel& accel) {
		Destroy();
		m_accels.push_back(accel);
	}

	void Clear() {
		Destroy();
		m_accels.clear();
	}

	bool GetHotKey(UINT cmd, VirtualKey& key, KeyModifiers& mods) const;
	UINT FindHotKey(VirtualKey key, KeyModifiers mods) const;
	void DecorateMenu(HMENU hMenu);
	void DoPersist(Persist& p);
};

//there should only be one of these for every menu
class CommandIdCreator {
private:
	const UINT mBase;
	UINT mCurNewId;
	vector<UINT> mOldIds;

	//Don't copy this thing. Pass it around by reference.
	CommandIdCreator(const CommandIdCreator&);
	CommandIdCreator& operator=(const CommandIdCreator&);

public:
	//initialize mBase to a high enough base value to not duplicate
	//anything the resource editor will generate
	CommandIdCreator() : mBase(40000), mCurNewId(mBase) {}

	UINT CreateId();
	void ReleaseId(UINT id);
	void ReleaseSubMenuItemsIds(HMENU hMenu);
};

class AutoBool {
private:
	bool mOldValue;
	bool& mBool;
public:

	AutoBool(bool& _bool, bool _value) : mBool(_bool) {
		mOldValue = mBool;
		mBool = _value;
	}

	~AutoBool() {mBool = mOldValue;}
};

class ModuleLoader {
private:
	HMODULE mModule;

public:
	ModuleLoader(LPCTSTR libFileName) {
		mModule = LoadLibrary(libFileName);
		ASSERT(mModule != NULL);
	}

	~ModuleLoader() {
		VERIFY(FreeLibrary(mModule));
	}
};

inline void DisplayError(const CString& msg, HWND hwnd = NULL) {
	MessageBox(hwnd, msg, ErrorStr, MB_OK|MB_ICONERROR);
}

typedef long ErrorCodes;
const ErrorCodes ErrNewerFileVersion = 1;
const ErrorCodes ErrApplicationBase = 10000;

//This is an exception class for exceptions thrown by the application framework.
//You can use this class for applicatation specific errors by inheriting from it,
//defining error codes starting at ErrApplicationBase, and overwritting the method
//Exception::GetErrorMessage(). The advantage of inheriting from this class for
//application specific errors is that the same catch can be used for application and
//framework errors.
class Exception {
public:
	Exception(ErrorCodes errorCode) : mErrorCode(errorCode) {}
	virtual ~Exception() {}

	const ErrorCodes mErrorCode;

	virtual CString GetErrorMessage() const;

	//this method should not be called if the error message needs arguments,
	//e.g. the message contains the format specifier %s.
	void DisplayDefaultError(HWND hwnd = NULL) const {
		DisplayError(GetErrorMessage(), hwnd);
	}
};

#endif