// LiteEditDlg.cpp : implementation file
//

#include "stdafx.h"
#include "LiteEdit.h"
#include "LiteEditDlg.h"
#include "GotoDlg.h"
#include "Persist.h"
#include "ToolsDlg.h"
#include "SyntaxColoringDlg.h"
#include "EditorOptionsDlg.h"
#include "HotkeysDlg.h"
#include "AboutDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static const int SBI_POS = 0;
static const int SBI_CLIPBOARD = 1;

static const UINT TID_EXTERNAL_MODIFY = 1;

//unfortunately, I have to hard code menu positions in
//since sub menus don't have an id
static const UINT MP_FILE = 0;
static const UINT MP_RECENT = 6;
static const UINT MP_TOOLS = 4;

/////////////////////////////////////////////////////////////////////////////
// CLiteEditDlg dialog

CLiteEditDlg::CLiteEditDlg(LPCTSTR fileNameArg, CWnd* pParent /*=NULL*/)
	: CDialog(CLiteEditDlg::IDD, pParent), mFileNameArg(fileNameArg)
{
	//{{AFX_DATA_INIT(CLiteEditDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLiteEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLiteEditDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLiteEditDlg, CDialog)
	//{{AFX_MSG_MAP(CLiteEditDlg)
	ON_WM_DESTROY()
	ON_WM_PAINT()
	ON_WM_CHANGECBCHAIN()
	ON_WM_DRAWCLIPBOARD()
	ON_WM_QUERYDRAGICON()
	ON_WM_SIZE()
	ON_WM_CLOSE()
	ON_WM_INITMENU()
	ON_CONTROL(ECN_CHANGE, ID_EDIT_CTRL, OnEditChange)
	ON_CONTROL(ECN_SELCHANGE, ID_EDIT_CTRL, OnEditSelChange)
	ON_REGISTERED_MESSAGE(WM_FINDREPLACE, OnFindReplace)
	ON_WM_TIMER()
	ON_WM_CANCELMODE()
	ON_WM_HELPINFO()
	ON_WM_QUERYENDSESSION()
	ON_WM_DROPFILES()
	ON_BN_CLICKED(ID_HELP, OnHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CLiteEditDlg::PreTranslateMessage(MSG* pMsg) {
	return
		TranslateAccelerator(m_hWnd, gConfigData.mAccelTable, pMsg) ||
		TranslateAccelerator(m_hWnd, gConfigData.mCommands.mAccelTable, pMsg);
}

//If user passed a file name, open it if it exists, otherwise, create a new file with
//that name. If no file name was passed, then create a new file with a default name.
void CLiteEditDlg::ProcessCommandLine() {
	CString path;

	if (FileExists(mFileNameArg)) {
		LPTSTR buff = path.GetBuffer(MAX_PATH);
		DWORD len = GetFullPathName(mFileNameArg, MAX_PATH, buff, NULL);
		len = GetLongPathName(buff, buff, MAX_PATH);
		path.ReleaseBuffer(len);
	} else
		path = mFileNameArg;

	bool newFile = !FileExists(path);
	if (path.IsEmpty())
		path = GetNewFileName();

	LoadFile(path, newFile);
}

//Loads fileName into mEdit. If newFile is true, then we create a newFile.
//If newFile is false, then we open an existing file. This parameter needs
//to be passed in because you could create a new file with the same same
//as an existing file, e.g. Untitiled.txt.
void CLiteEditDlg::LoadFile(const CString& fileName, bool newFile) {
	WaitCursor wc;

	bool exists = !newFile && FileExists(fileName);

	//display error message and don't do anything
	if (!newFile && !exists) {
		CString msg;
		msg.Format(TEXT("File '%s' does not exist."), fileName);
		MessageBox(msg, ErrorStr, MB_OK|MB_ICONERROR);
		gConfigData.mRecentFileList.Remove(fileName);
		return;
	}

	//Load the file text now before the text is modified or the file name is changed
	//because an exception could be thrown while trying to open the file. Lite Edit
	//shouldn't be an intermetiate state of loading the file if an exception is
	//thrown.
	CString fileText;
	if (!newFile)
		fileText = FileToString(fileName);

	mEdit.SetText(EmptyStr);

	//Setting the file name will set the language. Setting the text to empty
	//on the previous line prevents reparsing existing text as a different
	//language. Also, setting the language before loading the real text
	//prevents the real text from being parsed by the old language.
	mEdit.SetModified(false);//remove modify before title is set
	SetFileName(fileName);

	if (newFile)
		mEdit.SetReadOnly(false);
	else {
		ATLASSERT(exists);
		mEdit.SetText(fileText);
		mEdit.SetModified(false);
		bool readOnly = (GetFileAttributes(fileName) & FILE_ATTRIBUTE_READONLY) != 0;
		mEdit.SetReadOnly(readOnly);
	}

	UpdateTitle();//redraw title without modify marker and update readonly marker
	mEdit.EmptyUndoBuffer();
	SetFileInfo(exists);
}

//Saves text in mEdit to disk as fileName. Retruns false is save is canceled.
bool CLiteEditDlg::SaveFileAs(const CString& fileName) {
	DWORD attrs = GetFileAttributes(fileName);
	bool isDiskFileReadOnly = attrs != -1 && (attrs & FILE_ATTRIBUTE_READONLY) != 0;

	if (isDiskFileReadOnly) {
		CString msg;
		msg.Format(TEXT("You cannot save over read only file:\n%s\n\nYou must save this file with a different name or make the file on disk writeable."), fileName);
		MessageBox(msg, ErrorStr, MB_OK|MB_ICONERROR);
		return false;
	} else {
		WaitCursor wc;

		if (gConfigData.mRemoveTrailingWhitespaceOnSave)
			RemoveTrailingWhiteSpace();

		StringToFile(fileName, mEdit.GetTextEx(gConfigData.mNewLineType));
		mEdit.SetModified(false);
		SetFileName(fileName);
		SetFileInfo(true);
		return true;
	}
}

//Store last modified time of this file in mFileInfo. see OnTimer().
void CLiteEditDlg::SetFileInfo(bool exists) {
	mFileInfo.exists = exists;

	if (exists)
		mFileInfo.modifyTime = GetFileModifyTime(GetFileName());
}

//removes the trailing white space from each line in mEdit and
//adjusts the selection.
void CLiteEditDlg::RemoveTrailingWhiteSpace() {
	long fvl = mEdit.GetFirstVisibleLine();
	long fvp = mEdit.GetFirstVisiblePixel();
	long pos = 0;
	const long count = mEdit.GetLineCount();
	long anchor, activeEnd;
	mEdit.GetSel(anchor, activeEnd);

	CString text;
	LPTSTR textBuff = text.GetBuffer(mEdit.GetTextLength());
	LPTSTR pText = textBuff;

	for (long i = 0; i < count; ++i) {
		CString line = mEdit.GetLine(i);

		long len = line.GetLength();

		//this is basically an efficient way of
		//doing line.TrimRight(" \t")
		LPTSTR buff = line.GetBuffer(len);
		LPTSTR p = buff + len - 1;
		while (*p == ' ' || *p == '\t')
			--p;
		line.ReleaseBuffer(p - buff + 1);

		len -= line.GetLength();
		pos += line.GetLength();
		if (anchor > pos)
			anchor -= len;
		if (activeEnd > pos)
			activeEnd -= len;

		StrCpy(pText, line);
		pText += line.GetLength();

		//append a new line if it's not the last line
		if (i < count - 1) {
			*pText++ = '\n';
			++pos;
		}
	}

	int newLen = pText - textBuff;
	text.ReleaseBuffer(newLen);

	if (text.GetLength() != mEdit.GetTextLength()) {
		mEdit.SetText(text);
		mEdit.SetSel(anchor, activeEnd, skNone);
		mEdit.SetFirstVisibleLine(fvl);
		mEdit.SetFirstVisiblePixel(fvp);
	}
}

void CLiteEditDlg::SetLangFromFileExt() {
	LanguagePtr lang = gConfigData.mLangMgr.FindLanguage(GetFileExt(GetFileName()));
	mEdit.SetLanguage(lang);
}

void CLiteEditDlg::NewFile() {
	if (PromptToSave())
		LoadFile(GetNewFileName(), true);
}

CString CLiteEditDlg::GetNewFileName() const {
	CString title = TEXT("Untitled");
	AppendString(title, GetDefaultExt(), TEXT("."));
	return title;
}

//File>>Open command
void CLiteEditDlg::OpenFile() {
	if (PromptToSave()) {
		CString filter = gConfigData.mLangMgr.BuildFilter();
		CFileDialog dlg(TRUE, NULL, NULL, OFN_FILEMUSTEXIST|OFN_HIDEREADONLY, filter, this);
		dlg.m_ofn.nFilterIndex = gConfigData.mOpenFilterIndex;

		if (dlg.DoModal() == IDOK) {
			//Unforunately, I can't rely on CFileDialog() to add the default extension
			//because on Win 98 (at least) it will still add the default extension
			//if an extension is specified.
			CString path = dlg.GetPathName();
			if (!HasExt(path))
				AppendString(path, GetDefaultExt(), TEXT("."));

			LoadFile(path, false);
		}

		//store this for the next time we open a file
		gConfigData.mOpenFilterIndex = dlg.m_ofn.nFilterIndex;
	}
}

//Prompts user to save file if necessary and saves if told to. Returns false if the save
//is canceled.
bool CLiteEditDlg::PromptToSave() {
	if (mEdit.GetModified()) {
		int res = MessageBox(TEXT("Save modified file?"), TEXT("Save File"), MB_YESNOCANCEL|MB_ICONWARNING);
		if (res == IDYES)
			return Save();
		else
			return res != IDCANCEL;
	} else
		return true;
}

//File>>Save command. Returns false if save is canceled.
bool CLiteEditDlg::Save() {
	if (mEdit.GetModified()) {
		if (FileExists(GetFileName()))
			return SaveFileAs(GetFileName());
		else
			return SaveAs();
	}

	return true;
}

//File>>Save As command. Retruns false is save is canceled.
bool CLiteEditDlg::SaveAs() {
	CString filter = TEXT("ASCII Text|*.*||");
	CFileDialog dlg(FALSE, GetDefaultExt(), GetFileName(), OFN_OVERWRITEPROMPT, filter, this);
	if (dlg.DoModal() == IDOK) {
		return SaveFileAs(dlg.GetPathName());
	} else
		return false;
}

#ifdef PRINTING
#include "WinSpool.h"

void CLiteEditDlg::Print() {
	// get the default printer
	CPrintDialog dlg(FALSE);
	VERIFY(dlg.GetDefaults());

	CDC dc;
	dc.Attach(dlg.GetPrinterDC());

	DOCINFO info;
	InitSized(info);
	info.lpszDocName = GetFileName();
	VERIFY(dc.StartDoc(&info));
	VERIFY(dc.StartPage());

	dc.SetMapMode(MM_TEXT);

	int tabLength = mEdit.GetTabWidthInPixels();
	//dc.TabbedTextOut(0, 0, mEdit.GetText(), 1, &tabLength, 0);
	UINT flags = DT_EXPANDTABS|DT_EDITCONTROL|DT_LEFT|DT_NOPREFIX|DT_TABSTOP|DT_TOP;
	flags |= mEdit.GetTabWidth() << 8;
	CRect rc;
	rc.left = dc.GetDeviceCaps(PHYSICALOFFSETX);
	rc.top = dc.GetDeviceCaps(PHYSICALOFFSETY);
	rc.right = rc.left + dc.GetDeviceCaps(HORZRES);
	rc.bottom = rc.top + dc.GetDeviceCaps(VERTRES);

	dc.DrawText(mEdit.GetText(), &rc, flags);

	VERIFY(dc.EndPage());
	VERIFY(dc.EndDoc());
/*
	// is a default printer set up?
	HDC hdcPrinter = dlg.GetPrinterDC();
	if (hdcPrinter == NULL)
	{
	   MessageBox(_T("Buy a printer!"));
	}
	else
	{
	   // create a CDC and attach it to the default printer
	   CDC dcPrinter;
	   dcPrinter.Attach(hdcPrinter);

	   // call StartDoc() to begin printing
	   DOCINFO docinfo;
	   memset(&docinfo, 0, sizeof(docinfo));
	   docinfo.cbSize = sizeof(docinfo);
	   docinfo.lpszDocName = _T("CDC::StartDoc() Code Fragment");

	   // if it fails, complain and exit gracefully
	   if (dcPrinter.StartDoc(&docinfo) < 0)
	   {
		  MessageBox(_T("Printer wouldn't initalize"));
	   }
	   else
	   {
		  // start a page
		  if (dcPrinter.StartPage() < 0)
		  {
			 MessageBox(_T("Could not start page"));
			 dcPrinter.AbortDoc();
		  }
		  else
		  {
			 // actually do some printing
			 CGdiObject* pOldFont = dcPrinter.SelectStockObject(SYSTEM_FONT);

			 dcPrinter.TextOut(50, 50, _T("Hello World!"), 12);

			 dcPrinter.EndPage();
			 dcPrinter.EndDoc();
			 dcPrinter.SelectObject(pOldFont);
		  }
	   }
	}
*/
}
#endif

void CLiteEditDlg::SetFileName(const CString& name) {
	mFileName = name;
	SetLangFromFileExt();
	UpdateTitle();

	if (AbsolutePathExists(mFileName))
		gConfigData.mRecentFileList.Add(mFileName);
}

void CLiteEditDlg::UpdateTitle() {
	mHasModifiedMarker = false;
	CString title = GetFileName();//??? use path elipses

	if (mEdit.GetModified()) {
		title += '*';
		mHasModifiedMarker = true;
	}

	if (mEdit.GetReadOnly())
		title += TEXT(" [Read Only]");

	SetWindowText(title);
}

void CLiteEditDlg::DecorateAllMenus() {
	gConfigData.mAccelTable.DecorateMenu(*GetMenu());
	gConfigData.mAccelTable.DecorateMenu(mEdit.GetContextMenu());
	gConfigData.mCommands.mAccelTable.DecorateMenu(*GetMenu());
}

//calculates size of mEdit and mStatusBar
void CLiteEditDlg::ResizeControls() {
	//note that mStatusBar's height is set autoatically

	CRect rc;
	GetClientRect(rc);
	CRect newEditRc = rc;
	CRect newStatusRc(0, 0, 0, 0);
	if (::IsWindow(mStatusBar.m_hWnd))
		mStatusBar.GetClientRect(newStatusRc);
	newEditRc.bottom = Max(newEditRc.top, newEditRc.bottom - (long)newStatusRc.Height());
	newStatusRc.OffsetRect(rc.left, rc.bottom - newStatusRc.Height());
	newStatusRc.right = newStatusRc.left + rc.Width();

	if (::IsWindow(mStatusBar.m_hWnd))
		mStatusBar.MoveWindow(newStatusRc);

	if (mEdit.IsWindow()) {
		//I only want to invalidate what needs to be redrawn because there would be
		//a lot of flickering otherwise
		CRect oldRc;
		mEdit.GetClientRect(oldRc);
		mEdit.MoveWindow(newEditRc, FALSE);
		MapWindowPoints(CWnd::FromHandle(mEdit.m_hWnd), &newEditRc);

		CRgn oldRgn;
		oldRgn.CreateRectRgnIndirect(oldRc);
		CRgn newRgn;
		newRgn.CreateRectRgnIndirect(newEditRc);
		newRgn.CombineRgn(&newRgn, &oldRgn, RGN_DIFF);
		mEdit.InvalidateRgn(newRgn);
		mEdit.SendMessage(WM_NCPAINT, 1);//redraw non-client area
	}
}

//updates the pane that displays the row and column number of the caret
void CLiteEditDlg::UpdatePosPane() {
	CString text;

	long end = mEdit.GetSelCaret();
	long index, line;
	mEdit.LineFromChar(index, line, end);
	++line;//make 1 based
	long col = end - index + 1;//+1 to make 1 based
	text.Format(TEXT("Ln %d, Col %d"), line, col);

	mStatusBar.SetText(text, SBI_POS, 0);
}

//updates the pane that displays the clipboard
void CLiteEditDlg::UpdateClipboardPane() {
	CString clipboard;
	if (GetClipboardString(clipboard, m_hWnd, true)) {
		//I delete most of the characters from the clipboard because
		//mStatusBar.SetText() is slow for large text. Interestingly,
		//a status bar doesn't seem to support displaying more than 127
		//characters anyway!
		const long MostCharsNeeded = 500;
		if (clipboard.GetLength() > MostCharsNeeded)
			clipboard.Delete(MostCharsNeeded, clipboard.GetLength() - MostCharsNeeded);

		clipboard.Replace(TEXT("\n"), TEXT("\\n"));
		clipboard.Replace(TEXT("\t"), DupChar(' ', mEdit.GetTabWidth()));
	} else
		clipboard = TEXT("<Cannot get text from the clipboard>");

	mStatusBar.SetText(clipboard, SBI_CLIPBOARD, 0);
}

//Edit>>Advanced>>Insert Time/Date command.
void CLiteEditDlg::InsertTimeDate() {
	CString string = GetTimeDateString(gConfigData.mTimeDateFormat);
	mEdit.ReplaceSel(string);
}

//Edit>>Advanced>>UPPER CASE, lower case, and Title Case commands.
void CLiteEditDlg::ChangeSelCase(CaseKind kind) {
	CString sel = mEdit.GetSelText();

	switch (kind) {
	case ckLower :
		sel.MakeLower();
		break;
	case ckUpper :
		sel.MakeUpper();
		break;
	case ckTitle :
		MakeTitleCase(sel);
		break;
	case ckToggle :
		MakeToggleCase(sel);
		break;
	}

	mEdit.ReplaceSel(sel, true);//true to select replacment
}

//File>>Recent>>[recent file] and Tool>>[tool] commands.
bool CLiteEditDlg::ProcessDynamicCommand(UINT id) {
	CMenu* recentMenu = GetMenu()->GetSubMenu(MP_FILE)->GetSubMenu(MP_RECENT);
	int pos = FindMenuItemPosById(*recentMenu, id);

	if (0 <= pos && pos < gConfigData.mRecentFileList.GetItems().size()) {
		if (PromptToSave()) {
			//must copy file name because LoadFile() will modify
			//the recent file list which may change the file name unexpectedly
			CString fileName = gConfigData.mRecentFileList.GetItems()[pos];
			LoadFile(fileName, false);
		}

		return true;
	}

	Command* command = gConfigData.mCommands.FindCommand(id);
	if (command != NULL) {
		ExecuteCommand(*command);
		return true;
	}

	return false;
}

void CLiteEditDlg::ExecuteCommand(const Command& cmd) {
	bool cancel = cmd.mSaveFile && !Save();

	if (!cancel)
		gConfigData.Execute(cmd, GetFileName(), m_hWnd);
}

void CLiteEditDlg::ShowFindOrReplaceDialog(bool find) {
	if (IsWindow(mFindRelaceDlgPtr->GetSafeHwnd()))
		VERIFY(mFindRelaceDlgPtr->DestroyWindow());

	gConfigData.mFindReplaceData.selectionOnly = false;
	CString curWord;

	//If there's a selection use that as the current word, unless the selection spans
	//lines in which case there current word will be empty. If there is no selection,
	//use the word at the caret as the current word.
	//selectionOnly will be set to true if the selection spans lines.

	if (mEdit.HasSel()) {
		curWord = mEdit.GetSelText();
		if (curWord.Find('\n') >= 0) {//if selection spans lines
			curWord = EmptyStr;
			gConfigData.mFindReplaceData.selectionOnly = true;
		}
	} else {
		long start, end;
		mEdit.GetWordAt(mEdit.GetSelCaret(), start, end);
		curWord = mEdit.GetTextRange(start, end);
	}

	mFindRelaceDlgPtr = auto_ptr<FindReplaceDlg>(new FindReplaceDlg(find, curWord));
	mFindRelaceDlgPtr->Show(this);
}

void CLiteEditDlg::GetFindReplaceDlgOutOfTheWay() {
	if (IsWindow(mFindRelaceDlgPtr->GetSafeHwnd()) &&
		mFindRelaceDlgPtr->IsWindowVisible() &&
		mEdit.HasSel()) {
		long start, end;
		mEdit.GetSelRange(start, end);
		CPoint startPt = mEdit.PosFromChar(start);
		CPoint endPt = mEdit.PosFromChar(start);
		endPt.y += mEdit.GetLineHeight();
		mEdit.ClientToScreen(&startPt);
		mEdit.ClientToScreen(&endPt);

		CRect rc;
		mFindRelaceDlgPtr->GetWindowRect(&rc);

		long amountAbove = startPt.y - rc.bottom;
		long amountBelow = rc.top - endPt.y;

		if (amountAbove < 0 && amountBelow < 0) {//if dialog needs to be moved
			CRect screen = GetScreenRect();
			long moveUpY = (startPt.y - rc.Height()) / 2;
			long moveDownY = (screen.Height() + endPt.y - rc.Height()) / 2;

			if (moveUpY > screen.Height() - moveDownY - rc.Height())//if there's more room to move up
				rc.OffsetRect(0, moveUpY - rc.top);
			else
				rc.OffsetRect(0, moveDownY - rc.top);

			mFindRelaceDlgPtr->MoveWindow(&rc);
		}
	}
}

void CLiteEditDlg::DoFindOrReplace() {
	static bool lastSearchDown = true;

	//dlg can be either this window or the find/replace dialog. dlg should be the parent
	//of all dialogs displayed in this method.
	CWnd* dlg = IsWindow(mFindRelaceDlgPtr->GetSafeHwnd()) && mFindRelaceDlgPtr->IsWindowVisible() ? (CWnd*)mFindRelaceDlgPtr.get() : (CWnd*)this;
	FindReplaceData& data = gConfigData.mFindReplaceData;

	const CString& text = mEdit.GetText();
	bool matchCase = (data.flags & ffMatchCase) != 0;
	bool searchDown = (data.flags & ffReverse) == 0;

	if (data.replaceAll) {
		WaitCursor wc;
		long numReplacments = 0;

		if (data.selectionOnly) {
			CString selText = mEdit.GetSelText();
			numReplacments = ReplaceSubstring(selText, data.findStr, data.replaceStr, data.flags);
			if (numReplacments > 0)
				mEdit.ReplaceSel(selText, true);
		} else {
			long anchor = mEdit.GetSelAnchor();
			long activeEnd = mEdit.GetSelActiveEnd();
			long pos = 0;
			while (pos < text.GetLength()) {
				pos = FindSubstring(text, data.findStr, data.flags, pos);
				if (pos < 0)
					break;

				bool modify = mEdit.ModifyText(pos, data.findStr.GetLength(), data.replaceStr);
				if (modify) {
					long end = pos + data.findStr.GetLength();
					long lengthDiff = data.replaceStr.GetLength() - data.findStr.GetLength();
					if (anchor >= end)
						anchor += lengthDiff;
					if (activeEnd >= end)
						activeEnd += lengthDiff;

					pos += data.replaceStr.GetLength();
					++numReplacments;
				} else
					break;
			}

			mEdit.SetSel(anchor, activeEnd, skSelection);
		}

		CString msg;
		msg.Format(TEXT("%d replacments made."), numReplacments);
		dlg->MessageBox(msg, TEXT("Replace All"), MB_OK|MB_ICONINFORMATION);
	}

	if (data.replaceCurrent)
		if (StrCmpEx(mEdit.GetSelText(), data.findStr, matchCase) == 0)
			mEdit.ReplaceSel(data.replaceStr, true);

	if (data.findNext) {
		long end = searchDown ? mEdit.GetSelEnd() : mEdit.GetSelStart();
		long start = Min(end, (long)text.GetLength());
		long pos = text.IsEmpty() ? -1 : CyclicFindSubstring(text, data.findStr, data.flags, start, end);

		if (pos >= 0) {
			//reset mInitFindPos if direction of search has changed
			if (searchDown != lastSearchDown) {
				mInitFindPos = -1;
				lastSearchDown = searchDown;
			}

			//the next find is the same as the first find, so the user has searched
			//through all the text
			if (mInitFindPos == pos) {
				dlg->MessageBox(TEXT("You have finished searching through all the text for '") + data.findStr + TEXT("'."), TEXT("Finished Search"), MB_OK|MB_ICONINFORMATION);
				//all the user to search through the text again
				mInitFindPos = -1;
			} else {
				mLastFindPos = pos;
				if (mInitFindPos < 0)
					mInitFindPos = mLastFindPos;

				mEdit.SetSel(pos, pos + data.findStr.GetLength(), skSelection);
			}
		} else
			dlg->MessageBox(TEXT("There are no occurrences of '") + data.findStr + TEXT("'."), TEXT("Finished Search"), MB_OK|MB_ICONINFORMATION);
	}

	GetFindReplaceDlgOutOfTheWay();
}

void CLiteEditDlg::DoFindNextOrPrev(bool next) {
	FindReplaceData& data = gConfigData.mFindReplaceData;

	if (next)
		data.flags &= ~ffReverse;
	else
		data.flags |= ffReverse;

	if (!data.findStr.IsEmpty())
		DoFindOrReplace();
	else
		ShowFindOrReplaceDialog(true);
}

void CLiteEditDlg::BuildToolsMenu() {
	CMenu* toolsMenu = GetMenu()->GetSubMenu(MP_TOOLS);
	mIdCreator.ReleaseSubMenuItemsIds(*toolsMenu);

	HMENU newToolsMenu = LoadSubMenu(IDR_MENU, MP_TOOLS);
	gConfigData.mCommands.AddCommandsToMenu(newToolsMenu, mIdCreator);
	SetSubMenu(*GetMenu(), MP_TOOLS, newToolsMenu);
}

void CLiteEditDlg::DoGoto() {
	CGotoDlg dlg(mEdit.GetLineCount());
	if (dlg.DoModal() == IDOK) {
		long line = dlg.GetLineNumber() - 1;//- 1 to make zero-based
		line = Min(line, mEdit.GetLineCount() - 1L);
		line = Max(line, 0L);
		mEdit.GotoLine(line);
	}
}

void CLiteEditDlg::DisplayToolsDlg() {
	ToolsDlg dlg(GetFileName());
	if (dlg.DoModal() == IDOK)
		BuildToolsMenu();
}

void CLiteEditDlg::DisplayFontDlg() {
	LOGFONT lf;
	MemClear(lf);
	CFont& font = gConfigData.mFont;
	GetLogFont(font, lf);
	CFontDialog dlg(&lf, CF_FORCEFONTEXIST|CF_INITTOLOGFONTSTRUCT|CF_SCREENFONTS, NULL, this);

	if (dlg.DoModal() == IDOK) {
		dlg.GetCurrentFont(&lf);
		VERIFY(font.DeleteObject());
		font.CreateFontIndirect(&lf);
		mEdit.SetFont(font);
	}
}

void CLiteEditDlg::DisplayEditorOptionsDlg() {
	EditorOptionsDlg dlg;
	if (dlg.DoModal() == IDOK) {
		gConfigData.SetEdit(mEdit);
		UpdateClipboardPane();//tab width may have changed
	}
}

void CLiteEditDlg::DisplaySyntaxColoringDlg() {
	SyntaxColoringDlg dlg;
	//Apply config data even if user hits cancel because the user could have
	//applied changes (i.e. Apply button) and then canceled the dialog.
	dlg.DoModal();
	WaitCursor wc;
	gConfigData.SetEdit(mEdit);
	SetLangFromFileExt();
}

void CLiteEditDlg::DisplayHotKeysDlg() {
	//make File>>Recent empty so users can't configure hotkeys for
	//recent menu items
	CMenu* fileMenu = GetMenu()->GetSubMenu(MP_FILE);
	SetSubMenu(*fileMenu, MP_RECENT, CreatePopupMenu());

	HotKeysDlg dlg(*GetMenu());
	if (dlg.DoModal() == IDOK)
		DecorateAllMenus();
}

/////////////////////////////////////////////////////////////////////////////
// CLiteEditDlg message handlers

BOOL CLiteEditDlg::OnCommand(WPARAM wParam, LPARAM lParam) {
	WORD id = LOWORD(wParam), src = HIWORD(wParam);
	bool handled = true;

	if (src == 0 || src == 1) {//if from menu or accelerator
		switch (id) {
		case ID_FILE_NEW :
			NewFile();
			break;
		case ID_FILE_OPEN :
			OpenFile();
			break;
		case ID_FILE_SAVE :
			Save();
			break;
		case ID_FILE_SAVEAS :
			SaveAs();
			break;
#ifdef PRINTING
		case ID_FILE_PRINT :
			Print();
			break;
#endif
		case ID_FILE_EXIT :
			if (PromptToSave())
				PostQuitMessage(0);
			break;
		case ID_EDIT_UNDO :
			mEdit.Undo();
			break;
		case ID_EDIT_REDO :
			mEdit.Redo();
			break;
		case ID_EDIT_CUT :
			mEdit.Cut();
			break;
		case ID_EDIT_COPY :
			mEdit.Copy();
			break;
		case ID_EDIT_PASTE :
			mEdit.Paste();
			break;
		case ID_EDIT_DELETE :
			mEdit.ReplaceSel(EmptyStr);
			break;
		case ID_EDIT_SELECTALL :
			mEdit.SelectAll();
			break;
		case ID_EDIT_ADVANCED_SWITCHCLIPBOARD :
			gConfigData.SwitchClipboard(m_hWnd);
			break;
		case ID_EDIT_ADVANCED_INSERTDATETIME :
			InsertTimeDate();
			break;
		case ID_EDIT_ADVANCED_LOWERCASE :
			ChangeSelCase(ckLower);
			break;
		case ID_EDIT_ADVANCED_UPPERCASE :
			ChangeSelCase(ckUpper);
			break;
		case ID_EDIT_ADVANCED_TITLECASE :
			ChangeSelCase(ckTitle);
			break;
		case ID_EDIT_ADVANCED_TOGGLECASE :
			ChangeSelCase(ckToggle);
			break;
		case ID_SEARCH_FIND :
			ShowFindOrReplaceDialog(true);
			break;
		case ID_SEARCH_FINDNEXT :
			DoFindNextOrPrev(true);
			break;
		case ID_SEARCH_FINDPREVIOUS :
			DoFindNextOrPrev(false);
			break;
		case ID_SEARCH_REPLACE :
			ShowFindOrReplaceDialog(false);
			break;
		case ID_SEARCH_GOTO :
			DoGoto();
			break;
		case ID_BOOKMARKS_TOGGLE :
			mEdit.ToggleBookmark();
			break;
		case ID_BOOKMARKS_NEXT :
			mEdit.NextBookmark(true);
			break;
		case ID_BOOKMARKS_PREVIOUS :
			mEdit.NextBookmark(false);
			break;
		case ID_BOOKMARKS_CLEARALL :
			mEdit.ClearAllBookmarks();
			break;
		case ID_TOOLS_EDIT :
			DisplayToolsDlg();
			break;
		case ID_OPTIONS_FONT :
			DisplayFontDlg();
			break;
		case ID_OPTIONS_EDITOR :
			DisplayEditorOptionsDlg();
			break;
		case ID_OPTIONS_SYNTAXCOLORING :
			DisplaySyntaxColoringDlg();
			break;
		case ID_OPTIONS_HOTKEYS :
			DisplayHotKeysDlg();
			break;
		case ID_HELP_CONTENTS :
			gConfigData.ExecuteHelp(EmptyStr, m_hWnd);
			break;
		case ID_HELP_ABOUT :
			AboutDlg().DoModal();
			break;
		default :
			handled = ProcessDynamicCommand(id);
		}
	} else
		handled = false;

	return handled ? 1 : CBase::OnCommand(wParam, lParam);
}

BOOL CLiteEditDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	mInitFindPos = mLastFindPos = -1;

	DWORD style = WS_CHILD|WS_VISIBLE|WS_HSCROLL|WS_VSCROLL;
	DWORD exStyle = WS_EX_CLIENTEDGE;
	mEdit.Create(m_hWnd, CRect(0, 0, 0, 0), style, exStyle, ID_EDIT_CTRL);

	int widths[2] = {100, -1};
	VERIFY(mStatusBar.Create(WS_CHILD | WS_VISIBLE | CBRS_BOTTOM, CRect(0, 0, 0, 0), this, 0));
	VERIFY(mStatusBar.SetParts(sizeof(widths) / sizeof(int), widths));

	ResizeControls();

	UpdatePosPane();
	mNextClipViewer = SetClipboardViewer();

	gConfigData.Load();

	gConfigData.SetEdit(mEdit);

	HMENU editMenu = LoadSubMenu(IDR_MENU, 1);
	mEdit.SetContextMenu(editMenu);

	BuildToolsMenu();

	DecorateAllMenus();

	if (gConfigData.mMaximized) {
		//make sure window is maximized in the correct window for multi monitor
		SetWindowPos(NULL, gConfigData.mWinPos.left, gConfigData.mWinPos.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER);
		ShowWindow(SW_MAXIMIZE);
	} else if (!gConfigData.mWinPos.IsRectEmpty())
		MoveWindow(&gConfigData.mWinPos, FALSE);

	VERIFY(SetTimer(TID_EXTERNAL_MODIFY, 1000, NULL) == TID_EXTERNAL_MODIFY);

	DragAcceptFiles();

	ProcessCommandLine();

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CLiteEditDlg::OnDestroy() {
	VERIFY(KillTimer(TID_EXTERNAL_MODIFY));
	ChangeClipboardChain(mNextClipViewer);
	gConfigData.mMaximized = (Bool)IsZoomed();
	GetWindowRect(gConfigData.mWinPos);
	gConfigData.Save();
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLiteEditDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

void CLiteEditDlg::OnChangeCbChain(HWND hWndRemove, HWND hWndAfter) {
	if (mNextClipViewer == hWndRemove)
		mNextClipViewer = hWndAfter;
	::SendMessage(mNextClipViewer, WM_CHANGECBCHAIN, (WPARAM)hWndRemove, (LPARAM)hWndAfter);
}

void CLiteEditDlg::OnDrawClipboard() {
	UpdateClipboardPane();
	::SendMessage(mNextClipViewer, WM_DRAWCLIPBOARD, 0, 0);
}

HCURSOR CLiteEditDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CLiteEditDlg::OnSize(UINT nType, int cx, int cy) {
	CBase::OnSize(nType, cx, cy);
	ResizeControls();
}

void CLiteEditDlg::OnClose() {
	if (PromptToSave())
		CBase::OnClose();
}

void CLiteEditDlg::OnInitMenu(CMenu* menu) {
	ATLASSERT(menu != NULL && IsMenu(*menu));

	const bool hasSel = mEdit.HasSel();
	const bool textInClip = (Bool)IsClipboardFormatAvailable(CF_TEXT);
	const bool hasText = mEdit.GetTextLength() > 0;
	const bool isWriteable = mEdit.IsWriteable();

	//enable menu items only in the application menu
	if (menu == GetMenu()) {
		SetMenuItemEnabled(*menu, ID_FILE_SAVE, mEdit.GetModified());
		
		//*** dynamically create the File>>Recent menu ***

		CMenu* fileMenu = menu->GetSubMenu(MP_FILE);
		//release ids of menu that is about to be destroyed
		mIdCreator.ReleaseSubMenuItemsIds(*fileMenu->GetSubMenu(MP_RECENT));

		HMENU recentSubMenu = gConfigData.mRecentFileList.CreateMenu(mIdCreator);
		ASSERT(IsMenu(recentSubMenu));
		SetSubMenu(*fileMenu, MP_RECENT, recentSubMenu);
		bool enableRecent = GetMenuItemCount(recentSubMenu) > 0;
		SetMenuItemEnabled(*fileMenu, MP_RECENT, enableRecent, true);

		CString curExt = GetFileExt(GetFileName());
		for (size_t i = 0; i < gConfigData.mCommands.size(); ++i) {
			const Command& cmd = gConfigData.mCommands[i];
			bool enable = cmd.mExts.empty() || cmd.HasExtension(curExt);
			SetMenuItemEnabled(*menu, cmd.mAccel.cmd, enable);
		}
	}

	//enable menu items in both the application menu and context menu
	if (menu == GetMenu() || *menu == mEdit.GetContextMenu()) {
		SetMenuItemEnabled(*menu, ID_EDIT_UNDO, mEdit.CanUndo());
		SetMenuItemEnabled(*menu, ID_EDIT_REDO, mEdit.CanRedo());
		SetMenuItemEnabled(*menu, ID_EDIT_CUT, mEdit.CanCut());
		SetMenuItemEnabled(*menu, ID_EDIT_COPY, mEdit.CanCopy());
		SetMenuItemEnabled(*menu, ID_EDIT_PASTE, mEdit.CanPaste());
		SetMenuItemEnabled(*menu, ID_EDIT_DELETE, hasSel && isWriteable);
		SetMenuItemEnabled(*menu, ID_EDIT_SELECTALL, hasText);
		SetMenuItemEnabled(*menu, ID_EDIT_ADVANCED_SWITCHCLIPBOARD, textInClip);
		SetMenuItemEnabled(*menu, ID_EDIT_ADVANCED_INSERTDATETIME, isWriteable);
		SetMenuItemEnabled(*menu, ID_EDIT_ADVANCED_LOWERCASE, hasSel && isWriteable);
		SetMenuItemEnabled(*menu, ID_EDIT_ADVANCED_UPPERCASE, hasSel && isWriteable);
		SetMenuItemEnabled(*menu, ID_EDIT_ADVANCED_TITLECASE, hasSel && isWriteable);
		SetMenuItemEnabled(*menu, ID_EDIT_ADVANCED_TOGGLECASE, hasSel && isWriteable);
	}
}

void CLiteEditDlg::OnEditChange() {
	if (mHasModifiedMarker != mEdit.GetModified())
		UpdateTitle();
}

void CLiteEditDlg::OnEditSelChange() {
	UpdatePosPane();

	//if selected has changed because of anything but a find, reset the starting
	//find position
	if (mLastFindPos != mEdit.GetSelStart())
		mInitFindPos = -1;
}

LPARAM CLiteEditDlg::OnFindReplace(WPARAM wParam, LPARAM lParam) {
	DoFindOrReplace();
	return 0;
}

void CLiteEditDlg::OnTimer(UINT nIDEvent) 
{
	static bool inTimer = false;

	//check if readonly need to be updated or if the file needs to be reloaded
	if (nIDEvent == TID_EXTERNAL_MODIFY && mFileInfo.exists && !inTimer) {
		inTimer = true;

		DWORD attrs = GetFileAttributes(GetFileName());

		if (attrs == -1)//file doesn't exist. must have been deleted while being edited.
			mFileInfo.exists = false;
		else {
			bool readOnly = (attrs & FILE_ATTRIBUTE_READONLY) != 0;
			if (readOnly != mEdit.GetReadOnly()) {
				mEdit.SetReadOnly(readOnly);
				UpdateTitle();
			}

			FILETIME ft = GetFileModifyTime(GetFileName());
			if (CompareFileTime(&ft, &mFileInfo.modifyTime) > 0) {
				CString msg;
				msg.Format(TEXT("File '%s' has been modified externally. Do you want to reload it?"), GetFileName());
				
				if (MessageBox(msg, TEXT("Reload File"), MB_YESNO|MB_ICONQUESTION) == IDYES)
					LoadFile(GetFileName(), false);
				else
					mFileInfo.modifyTime = ft;//to keep from prompting again
			}
		}

		inTimer = false;
	}

	CDialog::OnTimer(nIDEvent);
}

//Overridden to keep error message from coming up. OnCommand() will display the help.
BOOL CLiteEditDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	return TRUE;
}

//I can get OnHelp() here if in a tab dialog you press F1 while focus in on a control
//that is not on a tab (e.g. OK, Cancel, and Help buttons). Since I don't want an
//error message to come up, I'll just show the main help page.
void CLiteEditDlg::OnHelp() {
	gConfigData.ExecuteHelp(EmptyStr, m_hWnd);	
}

BOOL CLiteEditDlg::OnQueryEndSession()
{
	return PromptToSave();
}

void CLiteEditDlg::OnDropFiles(HDROP hDropInfo) {
	if (PromptToSave()) {
		char fileName[MAX_PATH];
		VERIFY(DragQueryFile(hDropInfo, 0, fileName, MAX_PATH) > 0);	
		LoadFile(fileName, false);
	}
}