// SetupDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Setup.h"
#include "SetupDlg.h"
#include "ShlObj.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

LPCTSTR LiteEditDirName = TEXT("Lite Edit");

/////////////////////////////////////////////////////////////////////////////
// CSetupDlg dialog

CSetupDlg::CSetupDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSetupDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSetupDlg)
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSetupDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSetupDlg)
	DDX_Control(pDX, IDC_CHECK_SEND_TO, mCheckSendTo);
	DDX_Control(pDX, IDC_BUTTON_INSTALL, mBtnInstall);
	DDX_Control(pDX, IDC_EDIT_DIR, mEditDir);
	DDX_Control(pDX, IDC_CHECK_DESKTOP, mCheckDesktop);
	//}}AFX_DATA_MAP
}

void CSetupDlg::UpdateEnable() {
	mBtnInstall.EnableWindow(mEditDir.GetWindowTextLength() > 0);
}

BEGIN_MESSAGE_MAP(CSetupDlg, CDialog)
	//{{AFX_MSG_MAP(CSetupDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON_EXIT, OnButtonExit)
	ON_BN_CLICKED(IDC_BUTTON_BROWSE, OnButtonBrowse)
	ON_BN_CLICKED(IDC_BUTTON_INSTALL, OnButtonInstall)
	ON_EN_CHANGE(IDC_EDIT_DIR, OnChangeEditDir)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSetupDlg message handlers

BOOL CSetupDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	VERIFY(SUCCEEDED(CoInitialize(NULL)));

	mCheckDesktop.SetCheck(BST_CHECKED);
	mCheckSendTo.SetCheck(BST_CHECKED);

	CString dir = MakePath(GetProgramFilesPath(), LiteEditDirName);
	mEditDir.SetWindowText(dir);

	UpdateEnable();
		
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSetupDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CSetupDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CSetupDlg::OnButtonExit() 
{
	EndDialog(0);	
}

void CSetupDlg::OnButtonBrowse() 
{
	CString path;
	if (BrowseForFolder(m_hWnd, path)) {
		path = MakePath(path, LiteEditDirName);
		mEditDir.SetWindowText(path);
	}
}

void CSetupDlg::OnButtonInstall() 
{
	WaitCursor wc;

	StringVector created;

	CString dir;
	mEditDir.GetWindowText(dir);
	ASSERT(!dir.IsEmpty());

	if (!CreateDirPath(dir)) {
		CString msg;
		msg.Format(TEXT("Could not create directory '%s'."), dir);
		MessageBox(msg, ErrorStr, MB_OK|MB_ICONERROR);
		return;
	}

	CString errorMsg, temp;

	// *** Copy the files ***

	static TCHAR *fileList[] = {
		TEXT("LiteEdit.exe"),
		TEXT("LiteEditHelp.html"),
		TEXT("RealUninstall.exe"),
		TEXT("Uninstall.exe")};

	CString src, dest;
	CString moduleDir = GetModuleDir();
	const size_t NumFiles = sizeof(fileList) / sizeof(TCHAR*);
	for (size_t i = 0; i < NumFiles; ++i) {
		src = MakePath(moduleDir, fileList[i]);
		dest = MakePath(dir, fileList[i]);

		if (CopyFile(src, dest, FALSE))
			created.push_back(dest);
		else {
			temp.Format(TEXT("Could not install file '%s'\n"), fileList[i]);
			errorMsg += temp;
		}
	}

	StringVector langFileList;
	CString allLangFiles = MakePath(GetModuleDir(), TEXT("*.lang"));
	FindAllFiles(allLangFiles, langFileList, false);
	for (i = 0; i < langFileList.size(); ++i) {
		src = MakePath(moduleDir, langFileList[i]);
		dest = MakePath(dir, langFileList[i]);
		
		//pass TRUE to not overwrite existing language files
		CopyFile(src, dest, TRUE);
		created.push_back(dest);
	}

	//though not installed, since LiteEdit.ini will be created, it needs to be
	//kept track so it can be uninstalled
	created.push_back(MakePath(dir, TEXT("LiteEdit.ini")));

	// *** Create the shortcuts ***

	bool desktop = mCheckDesktop.GetCheck() == BST_CHECKED;
	bool sendTo = mCheckSendTo.GetCheck() == BST_CHECKED;

	LPCTSTR exeName = TEXT("LiteEdit.exe");
	CString exePath = MakePath(dir, exeName);
	LPCTSTR linkName = TEXT("Lite Edit.lnk");
	LPCTSTR desc = TEXT("Shortcut to Lite Edit");
	TCHAR shortcutDir[MAX_PATH];

	if (desktop) {
		GetSpecialFolderPath(CSIDL_DESKTOPDIRECTORY, shortcutDir);
		CString shortcutPath = MakePath(shortcutDir, linkName);
		if (SUCCEEDED(CreateLink(exePath, shortcutPath, desc)))
			created.push_back(shortcutPath);
		else
			errorMsg += TEXT("Could not create shortcut in Desktop\n");
	}

	if (sendTo) {
		GetSpecialFolderPath(CSIDL_SENDTO, shortcutDir);
		CString shortcutPath = MakePath(shortcutDir, linkName);
		if (SUCCEEDED(CreateLink(exePath, shortcutPath, desc)))
			created.push_back(shortcutPath);
		else
			errorMsg += TEXT("Could not create shortcut in Send To\n");
	}

	// *** Create program group ***

	GetSpecialFolderPath(CSIDL_PROGRAMS, shortcutDir);
	CString programGroupDir = MakePath(shortcutDir, TEXT("Lite Edit"));
	if (FileExists(programGroupDir) || CreateDirectory(programGroupDir, NULL)) {
		CString shortcutPath = MakePath(programGroupDir, linkName);		
		if (SUCCEEDED(CreateLink(exePath, shortcutPath, desc)))
			created.push_back(shortcutPath);
		else
			errorMsg += TEXT("Could not create Lite Edit shortcut in Program Group\n");

		shortcutPath = MakePath(programGroupDir, TEXT("Lite Edit Uninstaller.lnk"));
		CString uninstallerPath = MakePath(dir, TEXT("Uninstall.exe"));
		if (SUCCEEDED(CreateLink(uninstallerPath, shortcutPath, TEXT("Shortcut to Lite Edit uninstaller"))))
			created.push_back(shortcutPath);
		else
			errorMsg += TEXT("Could not create Lite Edit uninstaller shortcut in Program Group\n");

		created.push_back(programGroupDir);
	} else
		errorMsg += TEXT("Could not create Lite Edit Program Group\n");

	// *** Create the Uninstall.log file ***

	CString logPath = MakePath(dir, TEXT("Uninstall.log"));

	created.push_back(logPath);
	//push dir in last because directories can only be removed
	//if they're empty, i.e. after all other files in the
	//directory have been deleted
	created.push_back(dir);

	CString logData = created.Concat(TEXT("\r\n"));
	StringToFile(logPath, logData);

	// *** Display success or error message ***

	CString msg;
	UINT flags = MB_OK;

	if (errorMsg.IsEmpty()) {
		flags |= MB_ICONINFORMATION;
		msg = TEXT("Installation completed successfully!");
	} else {
		flags |= MB_ICONERROR;
		msg = TEXT("The following errors occurred during installation:\n\n") + errorMsg;
	}

	//msg += TEXT("\n\nTo uninstall Lite Edit choose:\nStart>>Programs>>Lite Edit>>Lite Edit Uninstaller");
	MessageBox(msg, TEXT("Finished"), flags);

	EndDialog(0);
}

void CSetupDlg::OnChangeEditDir() 
{
	UpdateEnable();
}

void CSetupDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
	CoUninitialize();	
}
